<?php

/**
 * This file is part of the eZ RepositoryForms package.
 *
 * @copyright Copyright (C) eZ Systems AS. All rights reserved.
 * @license For full copyright and license information view LICENSE file distributed with this source code.
 *
 * @version //autogentag//
 */
namespace EzSystems\RepositoryForms\Limitation;

/**
 * Interface for Limitation form mappers registry.
 */
interface LimitationFormMapperRegistryInterface
{
    /**
     * @return LimitationFormMapperInterface[]
     */
    public function getMappers();

    /**
     * Returns mapper corresponding to given Limitation identifier.
     *
     * @throws \InvalidArgumentException If no mapper exists for $limitationIdentifier.
     *
     * @return LimitationFormMapperInterface
     */
    public function getMapper($limitationIdentifier);

    /**
     * Checks if a mapper exists for given Limitation identifier.
     *
     * @param string $limitationIdentifier
     *
     * @return bool
     */
    public function hasMapper($limitationIdentifier);

    /**
     * @param LimitationFormMapperInterface $mapper
     * @param string $limitationIdentifier Limitation identifier the mapper is meant for.
     */
    public function addMapper(LimitationFormMapperInterface $mapper, $limitationIdentifier);
}
